#include <mockturtle-rt.h>

int main()
{
	int cpu, hmq, wait = 0, max_wait_cycle, count;
	const struct trtl_config_rom *cfgrom = trtl_config_rom_get();
	volatile uint32_t *msg_r, *msg_s;
	volatile struct trtl_hmq_header *hdr_r, *hdr_s;

	pr_debug("SYNC MESSAGES\r\n");
	cpu = 0;
	for (hmq = 0; hmq < cfgrom->n_hmq[cpu]; ++hmq)
		mq_purge(TRTL_HMQ, hmq);

	for (hmq = 0; hmq < cfgrom->n_hmq[cpu]; ++hmq) {
		wait = (1 << hmq);
		hdr_r = mq_map_in_header(TRTL_HMQ, hmq);
		msg_r = mq_map_in_buffer(TRTL_HMQ, hmq);

		hdr_s = mq_map_out_header(TRTL_HMQ, hmq);
		msg_s = mq_map_out_buffer(TRTL_HMQ, hmq);
		count = 1;
		max_wait_cycle = 0xFFFFFFFF;
		while (1) {
			/* Wait incoming message */
			while ((mq_poll_in(TRTL_HMQ) & wait) != wait) {
				if (--max_wait_cycle == 0) {
					pr_error("\tNO MESSAGE PENDING h:%d, cnt:0x%x\r\n",
						 hmq, count);
					return -1;
				}
			}

			/* validate message */
			if (hdr_r->len != 2 || msg_r[0] != count) {
				pr_error("\th: %d, l:%d, d[0]:0x%lx, cnt:0x%x\r\n",
					 hmq, hdr_r->len, msg_r[0], count);
				mq_discard(TRTL_HMQ, hmq);
				return -1;
			}
			if (msg_r[1] < count) {
				pr_error("\th:%d, l:%d d[1]:0x%lx, cnt:0x%x\r\n",
					 hmq, hdr_r->len, msg_r[1], count);
				mq_discard(TRTL_HMQ, hmq);
				return -1;
			}

			mq_claim(TRTL_HMQ, hmq);
			memcpy(hdr_s, hdr_r, sizeof(struct trtl_hmq_header));
			memcpy(msg_s, msg_r, (hdr_r->len + 1) * 4);

			mq_discard(TRTL_HMQ, hmq);
			mq_send(TRTL_HMQ, hmq);

			count++;
		}
	}

	return 0;
}
